@@ -1,4 +1,4 @@
-/* $Id: config.c 26397 2007-11-28 20:05:51Z benny $ */
+/* $Id: config.c 28292 2008-10-18 12:40:12Z kelnos $ */
 /*-
  * Copyright (c) 2003-2006 Benedikt Meurer <benny@xfce.org>
  * All rights reserved.
@@ -835,7 +835,7 @@ config_create (XfsmSplashRc *rc)
   GtkWidget         *page;
   GtkWidget         *swin;
   GtkWidget         *vbox;
-  const gchar       *theme;
+  gchar             *theme;
 
   theme = xfsm_splash_rc_read_entry (rc, "Theme", "Default");
 
@@ -914,6 +914,8 @@ config_create (XfsmSplashRc *rc)
   g_signal_connect (G_OBJECT (treeview), "destroy",
                     G_CALLBACK (config_store), rc);
 
+  g_free (theme);
+
   return page;
 }
 
@@ -947,7 +949,7 @@ config_configure (XfsmSplashConfig *config,
 static GdkPixbuf*
 config_preview (XfsmSplashConfig *config)
 {
-  const gchar *name;
+  gchar       *name;
   BalouTheme  *theme;
   GdkPixbuf   *pixbuf = NULL;
 
@@ -959,6 +961,8 @@ config_preview (XfsmSplashConfig *config)
   pixbuf = balou_theme_generate_preview (theme, 320, 240);
   balou_theme_destroy (theme);
 
+  g_free (name);
+
   return pixbuf;
 }
 
