@@ -1,4 +1,4 @@
-/* $Id: main.c 28142 2008-10-11 11:10:44Z kelnos $ */
+/* $Id: main.c 28615 2008-11-06 06:50:29Z kelnos $ */
 /*-
  * Copyright (c) 2003-2006 Benedikt Meurer <benny@xfce.org>
  * Copyright (c) 2008 Brian Tarricone <bjt23@cornell.edu>
@@ -137,11 +137,12 @@ init_display (XfsmManager   *manager,
               XfconfChannel *channel,
               gboolean       disable_tcp)
 {
-  const gchar *engine;
+  gchar *engine;
 
   engine = xfconf_channel_get_string (channel, "/splash/Engine", "mice");
 
   splash_screen = xfsm_splash_screen_new (dpy, engine);  
+  g_free (engine);
   xfsm_splash_screen_next (splash_screen, _("Loading desktop settings"));
 
   gdk_flush ();
